/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.client;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.NormalCloser;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.config.ConfigurationAuthority;
import de.bsvrz.dav.daf.main.config.ConfigurationException;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.client.dataobject.ProtocolAdministrationData;
import de.bsvrz.pua.prot.client.dataobject.ProtocolAggregationData;
import de.bsvrz.pua.prot.client.dataobject.ProtocolAggregationResultData;
import de.bsvrz.pua.prot.client.dataobject.ProtocolNoChanges;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResult;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResultData;
import de.bsvrz.pua.prot.data.ProtocolData;
import de.bsvrz.pua.prot.exceptions.ScopeException;
import de.bsvrz.pua.prot.util.ProcessingInformation;
import de.bsvrz.pua.prot.util.ProcessingParameter;
import de.bsvrz.pua.prot.util.ProtocolRequestResult;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.datk.AtlBase;
import de.bsvrz.sys.funclib.losb.datk.AtlDefaults;
import de.bsvrz.sys.funclib.losb.datk.AtlMeta;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;
import de.bsvrz.sys.funclib.losb.exceptions.LoggerException;
import de.bsvrz.sys.funclib.losb.util.Tuple;
import de.bsvrz.sys.funclib.losb.util.Util;

import java.io.*;
import java.nio.charset.Charset;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Kommandozeileninterface für 'Protokolle und Auswertungen'. Mit diesem Tool können Anfragen an die SWE Protokolle und Auswertungen gestellt werden.<br>
 * Kommandozeilenparameter:<br> <code> &nbsp;&nbsp;SkriptAnlegen<br> &nbsp;&nbsp;SkriptQuellcode<br> &nbsp;&nbsp;SkriptLoeschen<br>
 * &nbsp;&nbsp;ProtokollErstellen<br> &nbsp;&nbsp;ProtokollAbbrechen<br> &nbsp;&nbsp;ProtokollLoeschen<br> &nbsp;&nbsp;ProtokollStatus<br>
 * &nbsp;&nbsp;ProtokollAnhalten<br> &nbsp;&nbsp;ProtokollFortsetzen<br> &nbsp;&nbsp;GespeichertesProtokoll<br> &nbsp;&nbsp;UngelesenesProtokoll<br>
 * &nbsp;&nbsp;ListeGespeicherteProtokolle<br> &nbsp;&nbsp;ListeUngeleseneProtokolle<br> <br> OBLIGATORISCHE PARAMETER:<br> &nbsp;&nbsp;Operation=[String] -&gt;
 * die Methode, die aufgerufen wird (siehe MOEGLICHE OPERATIONEN)<br> &nbsp;&nbsp;KonfVerantwortlicher=[String] -&gt; PID des Konfigurationsverantwortlichen
 * (optional, Standard ist lokal Verantwortl.)<br> &nbsp;&nbsp;Host=[String] -&gt; Host auf dem der Datenverteiler laeuft<br> &nbsp;&nbsp;Port=[String] -&gt; Port des
 * Datenverteilers<br> &nbsp;&nbsp;Benutzer=[String] -&gt; Benutzername für Login beim Datenverteiler<br> &nbsp;&nbsp;Authentifizierung=[String] -&gt; Dateiname der
 * Datei, die Authentifikationsdaten für den Datenverteiler-Login enthaelt<br> &nbsp;&nbsp;Timeout=[Long] -&gt; Maximale Wartezeit in ms, bis eine Antwort
 * eingetroffen sein muss (optional)<br> &nbsp;&nbsp;Tabelle=[ja|nein] -&gt; Anzeige der Protokolle in einer Tabelle oder Ausgabe auf der Konsole.(optional-
 * Defaultwert 'nein')<br> &nbsp;&nbsp;Kopf=[ja|nein] -&gt; Der Kopf des Protokolls wird ausgegeben. (optional, Defaultwert 'ja')<br> <br> PARAMETER FUER
 * OPERATIONEN:<br> &nbsp;&nbsp;Datenarten=[1|2|3|4][,1|2|3|4]* -&gt; 1:Online, 2:Online-verzögert, 3:Nachgefprdert, 4:Nachgefordert-verzögert (kommagetrennt)<br>
 * &nbsp;&nbsp;Bindungen=[String]=[String]{,[String]=[String]}* -&gt; Bindungen der freien Aspekte<br> &nbsp;&nbsp;DeltaAnfrage=[ja|nein]<br>
 * &nbsp;&nbsp;Hauptobjekt=[PID des Hauptobjekts]<br> &nbsp;&nbsp;Objekte=[PIDs von anderen Objekten (kommagetrennt)]<br>
 * &nbsp;&nbsp;Zeitbereich=Long-Long[,Long-Long]* -&gt; Zeitbereiche<br> &nbsp;&nbsp;PseudoObjekte=([String]{,[String]}*n) = {([String]{,[String]}*n)}* -&gt;
 * Definitionen der Pseudoobjekte<br> &nbsp;&nbsp;Speichern=[ja|nein]<br> &nbsp;&nbsp;Skript=[PID des Skriptobjekts]<br> &nbsp;&nbsp;SimVar=[Short] -&gt;
 * Simulationsvariante<br> &nbsp;&nbsp;ProtokollId=[Long] -&gt; Id des Protokolls<br> &nbsp;&nbsp;Quelltext=[String] -&gt; Dateiname einer Datei, die den Quelltext
 * eines PuA-Skripts enthaelt<br> &nbsp;&nbsp;Name=[String] -&gt; Name des Skripts (optional fuer createScript)<br> &nbsp;&nbsp;Beschreibung=[String] -&gt;
 * Beschreibung (optional fuer createScript)<br> &nbsp;&nbsp;Autor=[String] -&gt; Author (optional fuer createScript)<br> &nbsp;&nbsp;Status=[String] -&gt; Status
 * (optional fuer createScript)<br> &nbsp;&nbsp;Datum=[String] -&gt; Datum (optional fuer createScript)<br> </code>
 *
 * @author beck et al. projects GmbH
 * @author Thomas Mueller
 * @version $Revision$ / $Date$ / ($Author$)
 */
@SuppressWarnings("deprecation")
public class CmdPua {

	/** Zeigt an, ob die Ausgabe mit {@link #getResultString()}, bzw. {@link #getStream()} verfügbar ist, oder einfach in System.out geschrieben wird. */
	private static boolean TEST = true;

	/** true: Anzeige der Protokolle in einer Tabelle false: Ausgabe auf der Konsole. */
	private boolean _gui = false;

	/** Ein Datenstrom (Rückgabe von {@link #callCreateProtocol()} */
	private ProtocolResultStream _stream;

	/** Ergebnis der letzten Operation als String */
	private String _resultString;

	/** Format für Datumseingabe festlegen. */
	protected static final SimpleDateFormat _sdf = new SimpleDateFormat("dd.MM.yyyyHH:mm:ss:SSS");	//yyyyHH OHNE Space ist korrekt!

	/** Id des zuletzt erstellten Protokolls. Nur für Testzwecke! */
	public static long mostRecentProtocolId = -1;

	/** Kopf wird ausgegeben */
	private boolean _printHeader = true;

	/**
	 * Main-Methode - Kann die Funktionen des PuA-Clients per Kommandozeile aufrufen (siehe JavaDocs zu {@link de.bsvrz.pua.prot.client.PuaClient}).<br> Muß mit
	 * korrekten Argumenten aufgerufen werden
	 *
	 * @param args die Argumente (s.o.)
	 *
	 * @throws Exception Exception
	 * @see #printInfo()
	 */
	public static void main(String[] args) throws Exception {
		Debug.setHandlerLevel("StdErr", Debug.WARNING);
		CmdPua.TEST = false;
		new CmdPua(args);
	}

	/////////////// Klassenvariablen und Methoden

	/** Kommandozeilen-Paramter */
	public HashMap<String, String> parameter = new HashMap<String, String>();

	/** Datenverteiler */
	public ClientDavInterface dav = null;

	/** Datenmodell */
	public DataModel model = null;

	/** Konfigurations-Authoritaet */
	public ConfigurationAuthority confAuth = null;

	/** Der PuA-Client */
	public PuaClient client = null;

	/** Ergebnis der letzen Operation */
	public ProtocolRequestResult requestResult = null;

	/**
	 * Zeigt an, ob die Datenverteilerverbindung beendet werden darf, oder nicht. Die Verbdinung bleibt bei Durchführung folgender Operationen (im Testbetrieb!)
	 * bestehen -ProtokollErstellen Bleibt sie aktiv, muss der Aufrufer die Verbindung beenden!
	 */
	private boolean stayAlive = false;

	/**
	 * Standard-Konstruktor: <br>Bei Benutzung diese Konstruktors muessen folgende Methoden manuell ausgefuehrt werden (in dieser Reihenfolge): <br> - {@link
	 * #setParameter(HashMap)} <br> - {@link #setDav()} <br> - {@link #setDataModel()} <br> - {@link #setConfigAuthority()} <br> - {@link #setClient()}
	 *
	 * @throws ConfigurationException Fehler bei der Kommunikation mit der Konfiguration.
	 * @throws ScopeException         Anfrage wird vom Konfigurationsverantwortlichen nicht unterstützt.
	 */
	public CmdPua() throws ConfigurationException, ScopeException {
	}

	/**
	 * Konstruktor mit Kommandozeilen-Parametern
	 *
	 * @param args Parameter
	 *
	 * @throws ConfigurationException Fehler bei der Kommunikation mit der Konfiguration.
	 * @throws ScopeException         Anfrage wird vom Konfigurationsverantwortlichen nicht unterstützt.
	 */
	public CmdPua(String args[]) throws ConfigurationException, ScopeException {
		// Parameter setzen
		for(String str : args) {
			String paramKey = str.split("=")[0].toLowerCase();
			parameter.put(paramKey, (str.split("=").length > 1) ? str.substring(str.split("=")[0].length() + 1) : "");
		}
		try {
			if(parameter.get("operation") != null && !parameter.get("operation").equals("")) {

				//Kopf anzeigen?
				if(this.parameter.get("kopf") != null) _printHeader = this.parameter.get("kopf").equals("ja");

				//Operation durchführen
				setDav();
				setDataModel();
				setConfigAuthority();
				setClient();

				try {
					this.callMethod();
				}
				catch(NumberFormatException e) {
					System.out.println("Fehler beim Konvertieren der Parameter (" + e.getMessage() + ")");
					requestResult = new ProtocolRequestResult(e.getMessage());
					this.printInfo();
				}
				catch(IllegalArgumentException illEx) {
					System.out.println("Ungueltiger Parameter (" + illEx.getMessage() + ")");
					requestResult = new ProtocolRequestResult(illEx.getMessage());
					this.printInfo();
				}
				catch(Exception e) {
					System.out.println("Fehler beim Abbarbeiten einer Operation (" + Util.getStackTrace(e) + ")");
					requestResult = new ProtocolRequestResult(Util.getStackTrace(e));
					this.printInfo();
				}
			}
			else {
				System.out.println("Keine Methode spezifiziert.");
				requestResult = new ProtocolRequestResult("Keine Methode angegeben");
				this.printInfo();
			}
		}
		catch(ConfigurationException confEx) {
			System.out.println("Ungueltige Konfiguration (" + confEx.getMessage() + ")");
			requestResult = new ProtocolRequestResult(confEx.getMessage());
			this.printInfo();
		}
		catch(NullPointerException npEx) {
			npEx.printStackTrace();
			System.out.println("Parameter sind wahrscheinlich falsch (" + npEx.getMessage() + ")");
			requestResult = new ProtocolRequestResult(npEx.getMessage());
			this.printInfo();
		}
		finally {
			this.cleanUp();
		}
	}

	/**
	 * Die Methode aufrufen, die im Parameter Operation angegeben wurde
	 *
	 * @throws NumberFormatException Exception
	 * @throws ConfigurationException Exception
	 * @throws FailureException Exception
	 * @throws IllegalArgumentException Exception
	 */
	public void callMethod() throws NumberFormatException, ConfigurationException, FailureException, IllegalArgumentException {

		// optional: Timeout setzen
		if(this.parameter.get("timeout") != null) {
			long timeout = new Long(this.parameter.get("timeout"));
			client.setTimeout(timeout);
		}
		//Ausgabeart setzen
		if(this.parameter.get("tabelle") != null) _gui = this.parameter.get("tabelle").equals("ja");

		String method = this.parameter.get("operation");
		// abortProtocol
		if(method.equalsIgnoreCase("ProtokollAbbrechen")) {
			_resultString = callAbortProtocol();
		}
		// createProtocol
		else if(method.equalsIgnoreCase("ProtokollErstellen")) {
			_stream = callCreateProtocol();
		}
		// createScript
		else if(method.equalsIgnoreCase("SkriptAnlegen")) {
			_resultString = callCreateScript();
		}
		// deleteSaveProtocol
		else if(method.equalsIgnoreCase("ProtokollLoeschen")) {
			_resultString = callDeleteSavedProtocol();
		}
		// deleteScript
		else if(method.equalsIgnoreCase("SkriptLoeschen")) {
			_resultString = callDeleteScript();
		}
		// getSavedProtocolList
		else if(method.equalsIgnoreCase("ListeGespeicherteProtokolle")) {
			_resultString = callGetSavedProtocolList();
		}
		// getSavedProtocol
		else if(method.equalsIgnoreCase("GespeichertesProtokoll")) {
			_stream = callGetSavedProtocol();
		}
		// getSavedProtocolList
		else if(method.equalsIgnoreCase("ListeUngeleseneProtokolle")) {
			_resultString = callGetUnreadProtocolList();
		}
		// getSavedProtocol
		else if(method.equalsIgnoreCase("UngelesenesProtokoll")) {
			_stream = callGetUnreadProtocol();
		}
		// getSource
		else if(method.equalsIgnoreCase("SkriptQuellcode")) {
			_resultString = callGetSource();
		}
		// getDefaults
		else if(method.equalsIgnoreCase("SkriptStandardwerte")) {
			_resultString = callGetDefaults();
		}
		// getStatus
		else if(method.equalsIgnoreCase("ProtokollStatus")) {
			_resultString = callGetStatus();
		}
		// pauseProtocol
		else if(method.equalsIgnoreCase("ProtokollAnhalten")) {
			_resultString = callSuspendProtocol();
		}
		// resumeProtocol
		else if(method.equalsIgnoreCase("ProtokollFortsetzen")) {
			_resultString = callResumeProtocol();
		}
		// listener
		else if(method.equalsIgnoreCase("ProtokollListener")) {
			_resultString = callProtocolListener();
		}
		// getPuAStatus
		else if(method.equalsIgnoreCase("Status")) {
			_resultString = callGetPuAStatus();
		}
		// method not specified
		else {
			System.out.println("Operation \"" + method + "\" ist nicht spezifiziert.");
		}
		printHeaderInfo("-----------------------------------------------------------------");
	}

	/**
	 * Listet Protokollfertigstellungen auf
	 * @return ""
	 */
	private String callProtocolListener() {
		stayAlive = true;
		System.out.println("FUEHRE \"ProtokollListener\" AUS:");
		try {
			client.addProtocolListener(new ProtocolListener(){
				public void protocolFinished(final long jobId) {
					System.out.println("Protokoll fertig: " + jobId);
				}
			});
		}
		catch(FailureException e) {
			e.printStackTrace();
		}
		return "";
	}

	/**
	 * Protokollerstellung abbrechen
	 *
	 * @return Ergebnis
	 */
	public String callAbortProtocol() {
		stayAlive = false;
		String retVal = "";
		System.out.println("FUEHRE \"ProtokollAbbrechen\" AUS:");
		try {
			System.out.println("\tErmittelte Parameter:");
			long protocolId = new Long(this.parameter.get("protokollid"));
			System.out.println("\t\t- ProtokollId = " + protocolId);

			requestResult = client.abortProtocol(protocolId);
			boolean erg = requestResult.isRequestSuccessful();
			retVal = "\tErfolgreich?\t" + (erg ? "Ja." : "Nein.");
			if(!erg) retVal += "\tFehler:\t\t" + requestResult.getErrorMsg();
		}
		catch(NullPointerException npEx) {
			retVal = "PARAMETERFEHLER:\n\tBenoetigte Parameter:" + "\n\t\t- ProtokollId";
			requestResult = new ProtocolRequestResult(retVal);
			npEx.printStackTrace();
		}
		if(!TEST) System.out.println(retVal);
		return retVal;
	}

	/**
	 * Protokoll erstellen (Prozessparameter fuellen und erstellen)
	 *
	 * @return der Datenstream
	 *
	 * @throws ConfigurationException Exception
	 * @throws IllegalArgumentException Exception
	 * @throws FailureException Exception
	 */
	public ProtocolResultStream callCreateProtocol() throws ConfigurationException, IllegalArgumentException, FailureException {
		printHeaderInfo("FUEHRE \"ProtokollErstellen\" AUS:");
		try {
			printHeaderInfo("\tErmittelte Parameter:");

			// Datenarten 1,2,3
			final String datenarten = this.parameter.get("datenarten");
			if(datenarten == null) throw new IllegalArgumentException("Argument Datenarten wurde nicht angegeben");
			String[] arKis = datenarten.split(",");
			ArchiveDataKind[] arKi = new ArchiveDataKind[arKis.length];
			for(int i = 0; (arKis != null && i < arKis.length); i++) {
				arKi[i] = ArchiveDataKind.getInstance(new Integer(arKis[i]));
			}
			String adks = "";
			for(ArchiveDataKind adk : arKi) {
				adks += adk.toString() + ", ";
			}
			printHeaderInfo("\t\t- Datenarten\t\t= " + adks.substring(0, adks.length() - 2));

			// bla=ex, blub=exs, ere=as
			String[] bind = (this.parameter.get("bindungen") == null) ? null : this.parameter.get("bindungen").split(",");
			Hashtable<String, String> aspBindings = new Hashtable<String, String>();
			for(int i = 0; (bind != null && i < bind.length); i++) {
				aspBindings.put(bind[i].split("=")[0].trim(), bind[i].split("=")[1].trim());
			}
			printHeaderInfo("\t\t- Bindungen\t\t= " + aspBindings);

			Boolean deltaProtocol = null;
			if(this.parameter.get("deltaanfrage") != null) {
				deltaProtocol = (this.parameter.get("deltaanfrage").equalsIgnoreCase("ja"));
				printHeaderInfo("\t\t- DeltaAnfrage\t\t= " + (deltaProtocol ? "ja" : "nein"));
			}

			// (123-234) (234-345)
			List<Tuple<Long, Long>> periodList = new ArrayList<Tuple<Long, Long>>();
			String periods = this.parameter.get("zeitbereich");
			if(periods != null) {
				String[] pList = periods.replace(" ", "").split(",");
				for(int i = 0; (pList != null && i < pList.length); i++) {
					String first = pList[i].split("-")[0];
					String last = pList[i].split("-")[1];

					//unterscheiden zwischen Angabe im ms-Format oder ähnlich wie im Protokoll selbst
					try {
						periodList.add(new Tuple<Long, Long>(new Long(first), new Long(last)));
					}
					catch(NumberFormatException nfe) {
						try {
							GregorianCalendar gc = new GregorianCalendar();
							gc.setTime(_sdf.parse(first));
							long start = gc.getTimeInMillis();
							gc.clear();
							gc.setTime(_sdf.parse(last));
							long end = gc.getTimeInMillis();
							periodList.add(new Tuple<Long, Long>(start, end));
						}
						catch(ParseException e) {
							throw new FailureException(
									"Zeitbereich wurde fehlerhaft angegeben: Erwartetes Format: dd.MM.yyyy HH:mm:ss:SSS.", LoggerException.ERROR
							);
						}
					}
				}
				printHeaderInfo("\t\t- Zeitbereich\t\t= " + periodListToString(periodList));
			}

			// (alias1,alias2) = (def1_1,def2_1) (def1_2, def2_2)
			// -> tmp[0] = (alias1,alias2)
			// -> tmp[1] = (def_1,def_2)(def_1,def_2)
			Hashtable<String, String[]> pseudoObjects = new Hashtable<String, String[]>();
			String pseuObj = (this.parameter.get("pseudoobjekte") == null) ? "" : this.parameter.get("pseudoobjekte");
			if(!pseuObj.equals("")) {
				String[] tmp = pseuObj.replace(" ", "").split("=");
				// (alias1,alias2)
				// -> aliasTmp[0] = alias1
				// -> aliasTmp[1] = alias2
				String[] aliasTmp = tmp[0].trim().substring(1, tmp[0].length() - 1).split(",");
				// (def1_1,def2_1)(def1_2,def2_2)
				// -> def[0] = def1_1,def2_1
				// -> def[1] = def2_1,def2_2
				String[] def = tmp[1].trim().substring(1, tmp[1].length() - 1).split("\\)\\(");
				// hashtable initialisieren
				for(String alias : aliasTmp) {
					pseudoObjects.put(alias, new String[def.length]);
				}
				for(int i = 0; i < def.length; i++) {
					String[] defSingle = def[i].split(",", aliasTmp.length);
					for(int j = 0; j < aliasTmp.length; j++) {
						String[] objOfAlias = pseudoObjects.get(aliasTmp[j]);
						objOfAlias[i] = defSingle[j];
						pseudoObjects.put(aliasTmp[j], objOfAlias);
					}
				}
				String psObjStr = "\t\t- PseudoObjekte\t\t= [" + hash2String(pseudoObjects) + "]";
				printHeaderInfo(psObjStr);
			}

			Boolean saveProtocol = null;
			if(this.parameter.get("speichern") != null) {
				saveProtocol = (this.parameter.get("speichern").equalsIgnoreCase("ja"));
				printHeaderInfo("\t\t- Speichern\t\t= " + (saveProtocol ? "ja" : "nein"));
			}

			DynamicObject script = null;
			if(parameter.get("skript") != null) {
				if(model.getObject(parameter.get("skript")) instanceof DynamicObject) {
					script = (DynamicObject)model.getObject(parameter.get("skript"));
				}
				else {
					throw new IllegalArgumentException("Skript mit der PID " + parameter.get("skript") + " existiert nicht.");
				}
			}
			printHeaderInfo("\t\t- Skript\t\t= " + ((script != null) ? script.getPid() : "kein Skript"));

			final String mainObjectPid = this.parameter.get("hauptobjekt");
			final SystemObject mainObject;
			if(mainObjectPid != null){
				mainObject = model.getObject(mainObjectPid);
				printHeaderInfo("\t\t- Hauptobjekt\t\t= " + mainObject);
			}
			else{
				mainObject = null;
			}

			short simVar = new Short(this.parameter.get("simvar"));
			printHeaderInfo("\t\t- Simulationsvariante\t= " + simVar);

			ProcessingParameter pp = new ProcessingParameter(script, mainObject, simVar);
			pp.setArchiveDataKind(arKi);
			pp.setAspBindings(aspBindings);
			pp.setDeltaProtocol(deltaProtocol);
			pp.setPeriodList(periodList);
			pp.setPseudoObjects(pseudoObjects);
			pp.setSaveProtocol(saveProtocol);
			pp.setCreationDate(System.currentTimeMillis());
			String creator = parameter.get("creator");
			if(creator == null) creator = "";
			pp.setCreatorName(creator);
			String info = parameter.get("info");
			if(info == null) info = "";
			pp.setInfoText(info);
			ProtocolResultStream protocolResultStream = client.createProtocol(pp);
			stayAlive = TEST;
			if(!TEST) {
				printStream(protocolResultStream);
			}
			return protocolResultStream;
		}
		catch(NullPointerException npEx) {
			requestResult = new ProtocolRequestResult(npEx.getMessage());
			if(!TEST) {
				System.out.println("PARAMETERFEHLER:");
				System.out.println(
						"\tBenoetigte Parameter:" + "\n\t\t- Datenarten" + "\n\t\t- Bindungen" + "\n\t\t- DeltaAnfrage" + "\n\t\t- Objekte"
						+ "\n\t\t- Zeitbereich" + "\n\t\t- PseudoObjekte" + "\n\t\t- Speichern" + "\n\t\t- Hauptobjekt" + "\n\t\t- Skript" + "\n\t\t- SimVar"
				);
			}
			npEx.printStackTrace();
			return null;
		}
	}

	/**
	 * Gibt den Header aus - falls vom User gewünscht.
	 *
	 * @param string
	 */
	private void printHeaderInfo(String string) {
		if(_printHeader) System.out.println(string);
	}

	private String periodListToString(List<Tuple<Long, Long>> periodList) {
		StringBuffer sb = new StringBuffer();
		if(periodList != null) {
			for(int i = 0; i < periodList.size(); i++) {
				Tuple<Long, Long> p = periodList.get(i);
				if(i > 0) sb.append(", ");
				Util.msToDate(sb, p.first);
				sb.append(" - ");
				Util.msToDate(sb, p.last);
			}
		}
		return sb.toString();
	}

	/**
	 * Skript erstellen
	 *
	 * @return Ergebnis
	 *
	 * @throws ConfigurationException Exception
	 * @throws IllegalArgumentException Exception
	 */
	public String callCreateScript() throws ConfigurationException, IllegalArgumentException {
		stayAlive = false;
		String retVal = "";
		System.out.println("FUEHRE \"SkriptAnlegen\" AUS:");
		String createdScript = "";
		try {
			System.out.println("\tErmittelte Parameter:");

			String pid = this.parameter.get("skript");
			System.out.println("\t\t- Skript\t= " + (("".equals(pid)) ? "nicht angegeben" : pid));

			String name = (this.parameter.get("name") == null) ? "" : this.parameter.get("name");
			System.out.println("\t\t- Name\t\t= " + (("".equals(name)) ? "nicht angegeben" : name));

			String description = (this.parameter.get("beschreibung") == null) ? "" : this.parameter.get("beschreibung");
			System.out.println("\t\t- Beschreibung\t= " + (("".equals(description)) ? "nicht angegeben" : description));

			String author = (this.parameter.get("autor") == null) ? "" : this.parameter.get("autor");
			System.out.println("\t\t- Autor\t\t= " + (("".equals(author)) ? "nicht angegeben" : author));

			String status = (this.parameter.get("status") == null) ? "" : this.parameter.get("status");
			System.out.println("\t\t- Status\t= " + (("".equals(status)) ? "nicht angegeben" : status));

			String date = (this.parameter.get("datum") == null) ? "" : this.parameter.get("datum");
			System.out.println("\t\t- Datum\t\t= " + (("".equals(date)) ? "nicht angegeben" : date));

			AtlBase baseInfo = new AtlBase(name, description, author, status, date);

			//  Skriptquelltext aus der angebenen Datei lesen
			String source = "";
			if(this.parameter.get("quelltext") != null) {
				BufferedReader sourceFile = new BufferedReader(new InputStreamReader(new FileInputStream(parameter.get("quelltext")), Charset.forName("ISO-8859-1")));
				String line;
				while((line = sourceFile.readLine()) != null) {
					source += (line + "\n");
				}
			}
			System.out.println("\t\t- Quelltext\t= \n" + source + "\n");

			//Skriptobjekt anlegen
			requestResult = client.createScript(pid, name, baseInfo, source);
			createdScript = client.recentlyCreatedScript.getPid();
			boolean erg = requestResult.isRequestSuccessful();
			retVal = "\tErfolgreich?\t" + (erg ? "Ja." : "Nein.") + "\n" + ((erg)
			                                                                ? ("\tPID des Skripts:\t\"" + createdScript + "\"")
			                                                                : ("\tFehler:\t\t" + requestResult.getErrorMsg()));
		}
		catch(NullPointerException npEx) {
			retVal = "PARAMETERFEHLER:" + "\n\tBenoetigte Parameter:" + "\n\t\t- Quelltext" + "\n\t\t- Skript (optional)" + "\n\t\t- Name (optional)"
			         + "\n\t\t- Beschreibung (optional)" + "\n\t\t- Autor (optional)" + "\n\t\t- Status (optional)" + "\n\t\t- Datum (optional)";
			requestResult = new ProtocolRequestResult(retVal);
		}
		catch(FileNotFoundException fnfEx) {
			retVal = "PARAMETERFEHLER:\n\tQuelltextdatei " + this.parameter.get("quelltext") + " existiert nicht.";
			requestResult = new ProtocolRequestResult(retVal);
		}
		catch(IOException ioEx) {
			retVal = "\n\tEin-/Ausgabefehler beim Lesen der Quelltextdatei " + this.parameter.get("quelltext") + ".";
			requestResult = new ProtocolRequestResult(retVal);
		}
		if(!TEST) System.out.println(retVal);
		return createdScript;
	}

	/**
	 * Loescht ein gespeichertes Protokoll.
	 *
	 * @return Ergebnis
	 */
	public String callDeleteSavedProtocol() {
		stayAlive = false;
		String retVal = "";
		System.out.println("FUEHRE \"ProkollLoeschen\" AUS:");
		try {
			System.out.println("\tErmittelte Parameter:");

			long protocolId = new Long(this.parameter.get("protokollid"));
			System.out.println("\t\t- ProtokollId = " + protocolId);

			requestResult = client.deleteSaveProtocol(protocolId);
			boolean erg = requestResult.isRequestSuccessful();
			retVal = "\tErfolgreich?\t" + (erg ? "Ja." : "Nein.");
			if(!erg) retVal += "\tFehler:\t\t" + requestResult.getErrorMsg();
		}
		catch(NullPointerException npEx) {
			retVal = ("PARAMETERFEHLER:\n\tBenoetigte Parameter:\n\t\t- ProtokollId");
			requestResult = new ProtocolRequestResult(retVal);
		}
		if(!TEST) System.out.println(retVal);
		return retVal;
	}

	/**
	 * Skript loeschen
	 *
	 * @return Ergebnis
	 *
	 * @throws ConfigurationException Exception
	 * @throws IllegalArgumentException Exception
	 */
	public String callDeleteScript() throws ConfigurationException, IllegalArgumentException {
		stayAlive = false;
		String retVal = "";
		System.out.println("FUEHRE \"SkriptLoeschen\" AUS:");
		try {
			System.out.println("\tErmittelte Parameter:");
			DynamicObject script = null;
			if(parameter.get("skript") != null) {
				if(model.getObject(parameter.get("skript")) instanceof DynamicObject) {
					script = (DynamicObject)model.getObject(parameter.get("skript"));
				}
				else {
					throw new IllegalArgumentException("Skript mit der PID " + parameter.get("skript") + " existiert nicht.");
				}
			}
			System.out.println("\n\t\t- Skript = " + script.getPid());

			requestResult = client.deleteScript(script);
			boolean erg = requestResult.isRequestSuccessful();
			retVal = "\tErfolgreich?\t" + (erg ? "Ja." : "Nein.");
			if(!erg) retVal += "\tFehler:\t\t" + requestResult.getErrorMsg();
		}
		catch(NullPointerException npEx) {
			retVal = ("PARAMETERFEHLER:\n\tBenoetigte Parameter:\n\t\t- Skript");
			requestResult = new ProtocolRequestResult(retVal);
		}
		if(!TEST) System.out.println(retVal);
		return retVal;
	}

	/**
	 * Gespeichertes Protokoll abrufen
	 *
	 * @return Ergebnis
	 */
	public ProtocolResultStream callGetSavedProtocol() {
		String retVal = "";
		printHeaderInfo("FUEHRE \"GespeichertesProtokoll\" AUS:");
		try {
			System.out.println("\tErmittelte Parameter:");

			long protocolId = Long.parseLong(parameter.get("protokollid"));
			printHeaderInfo("ProtokollId = " + protocolId);

			_stream = client.getSavedProtocol(protocolId);
			stayAlive = TEST;
			if(!TEST) printStream(_stream);
			return _stream;
		}
		catch(NullPointerException npEx) {
			retVal = "PARAMETERFEHLER:\n\tBenoetigte Parameter:" + "\n\t\t- ProtokollId";
			if(!TEST) System.out.println(retVal);
			requestResult = new ProtocolRequestResult(retVal);
			return null;
		}
		catch(FailureException fEx) {
			retVal = "PARAMETERFEHLER:\n\tProtokoll scheint nicht vorhanden zu sein.";
			if(!TEST) System.out.println(retVal);
			requestResult = new ProtocolRequestResult(retVal);
			return null;
		}
	}

	/**
	 * Ungelesenes Protokoll abrufen
	 *
	 * @return Ergebnis
	 */
	public ProtocolResultStream callGetUnreadProtocol() {
		String retVal = "";
		System.out.println("FUEHRE \"UngelesenesProtokoll\" AUS:");
		try {
			System.out.println("\tErmittelte Parameter:");

			long protocolId = Long.parseLong(parameter.get("protokollid"));
			System.out.println("\t\t- ProtokollId = " + protocolId);

			_stream = client.getUnreadProtocol(protocolId);
			stayAlive = TEST;
			if(!TEST) printStream(_stream);
			return _stream;
		}
		catch(NullPointerException npEx) {
			retVal = "PARAMETERFEHLER:\n\tBenoetigte Parameter:" + "\n\t\t- ProtokollId";
			if(!TEST) System.out.println(retVal);
			requestResult = new ProtocolRequestResult(retVal);
			return null;
		}
		catch(FailureException fEx) {
			retVal = "PARAMETERFEHLER:\n\tProtokoll scheint nicht vorhanden zu sein.";
			if(!TEST) System.out.println(retVal);
			requestResult = new ProtocolRequestResult(retVal);
			return null;
		}
	}

	/**
	 * Liste der gespeicherten Protokolle abrufen
	 *
	 * @return Ergebnis
	 */
	public String callGetSavedProtocolList() {
		stayAlive = false;
		String retVal = "";
		System.out.println("FUEHRE \"ListeGespeicherteProtokolle\" AUS:");
		System.out.println("\tErmittelte Parameter:\n\t\tKeine Parameter benoetigt.");

		requestResult = new ProtocolRequestResult();
		Tuple<Long, ProcessingParameter>[] savedProtocolList = client.getSavedProtocolList(requestResult, ""); 
		boolean erg = requestResult.isRequestSuccessful();
		retVal = "\tErfolgreich?\t" + (erg ? "Ja." : "Nein.");
		if(erg) {
			retVal += "\n\tGespeicherte Protokolle:";
			for(int i = 0; (i < savedProtocolList.length); i++) {
				retVal += "\n\t\tId: " + savedProtocolList[i].first + " (" + savedProtocolList[i].last.toString() + ")";
			}
		}
		else {
			retVal += "\n\tFehler:\t\t" + requestResult.getErrorMsg();
		}

		if(!TEST) System.out.println(retVal);
		return retVal;
	}


	/**
	 * Liste unglesener Protokolle abrufen
	 *
	 * @return Ergebnis
	 */
	public String callGetUnreadProtocolList() {
		stayAlive = false;
		String retVal = "";
		System.out.println("FUEHRE \"ListeUngeleseneProtokolle\" AUS:");
		System.out.println("\tErmittelte Parameter:\n\t\tKeine Parameter benoetigt.");

		requestResult = new ProtocolRequestResult();
		Tuple<Long, ProcessingParameter>[] savedProtocolList = client.getUnreadProtocolList(requestResult, ""); 
		boolean erg = requestResult.isRequestSuccessful();
		retVal = "\tErfolgreich?\t" + (erg ? "Ja." : "Nein.");
		if(erg) {
			retVal += "\nUngelesene Protokolle:";
			for(int i = 0; (i < savedProtocolList.length); i++) {
				retVal += "\n\t\tId: " + savedProtocolList[i].first + " (" + savedProtocolList[i].last.toString() + ")";
			}
		}
		else {
			retVal += "\n\tFehler:\t\t" + requestResult.getErrorMsg();
		}

		if(!TEST) System.out.println(retVal);
		return retVal;
	}

	/**
	 * Quelltext zu einem Skript
	 *
	 * @return Ergebnis
	 *
	 * @throws ConfigurationException Exception
	 * @throws IllegalArgumentException Exception
	 */
	public String callGetSource() throws ConfigurationException, IllegalArgumentException {
		stayAlive = false;
		String retVal = "";
		System.out.println("FUEHRE \"SkriptQuelltext\" AUS:");
		try {
			System.out.println("\tErmittelte Parameter:");
			DynamicObject script = null;
			if(parameter.get("skript") != null) {
				if(model.getObject(parameter.get("skript")) instanceof DynamicObject) {
					script = (DynamicObject)model.getObject(parameter.get("skript"));
				}
				else {
					throw new IllegalArgumentException("Skript mit der PID " + parameter.get("skript") + " existiert nicht.");
				}
			}
			System.out.println("\t\t- Skript = " + script.getPid());

			StringBuffer buffer = new StringBuffer();
			AtlMeta meta = new AtlMeta();
			requestResult = client.getSource(script, buffer, meta);
			boolean erg = requestResult.isRequestSuccessful();
			retVal = "\tErfolgreich?\t" + (erg ? "Ja." : "Nein.") + ((erg)
			                                                         ? ("\n---\nSkript Details:\n" + meta + "\n---\nQuelltext:\n" + buffer)
			                                                         : ("\n\tFehler:\n\t\t" + requestResult.getErrorMsg()));
		}
		catch(NullPointerException npEx) {
			retVal = "PARAMETERFEHLER:\n\tBenoetigte Parameter:" + "\n\t\t- Skript";
			requestResult = new ProtocolRequestResult(retVal);
		}
		catch(ClassCastException ccEx) {
			retVal = "PARAMETERFEHLER:\n\tDas angegebene Skript existiert nicht.";
			requestResult = new ProtocolRequestResult(retVal);
		}
		if(!TEST) System.out.println(retVal);
		return retVal;
	}

	/**
	 * Standardwerte zu einem Skript
	 *
	 * @return Ergebnis
	 *
	 * @throws ConfigurationException Exception
	 * @throws IllegalArgumentException Exception
	 */
	public String callGetDefaults() throws ConfigurationException, IllegalArgumentException {
		stayAlive = false;
		String retVal = "";
		System.out.println("FUEHRE \"Standardwerte\" AUS:");
		try {
			System.out.println("\tErmittelte Parameter:");
			DynamicObject script = null;
			if(parameter.get("skript") != null) {
				if(model.getObject(parameter.get("skript")) instanceof DynamicObject) {
					script = (DynamicObject)model.getObject(parameter.get("skript"));
				}
				else {
					throw new IllegalArgumentException("Skript mit der PID " + parameter.get("skript") + " existiert nicht.");
				}
			}
			System.out.println("\t\t- Skript = " + script.getPid());

			AtlDefaults defaults = new AtlDefaults();
			AtlMeta meta = new AtlMeta();
			requestResult = client.getDefaults(script, defaults, meta);
			boolean erg = requestResult.isRequestSuccessful();
			retVal = "\tErfolgreich?\t" + (erg ? "Ja." : "Nein.") + ((erg)
			                                                         ? ("\n---\nSkript Details:\n" + meta + "\n---\nStandardwerte:\n" + defaults)
			                                                         : ("\n\tFehler:\n\t\t" + requestResult.getErrorMsg()));
		}
		catch(NullPointerException npEx) {
			retVal = "PARAMETERFEHLER:\n\tBenoetigte Parameter:" + "\n\t\t- Skript";
			requestResult = new ProtocolRequestResult(retVal);
		}
		catch(ClassCastException ccEx) {
			retVal = "PARAMETERFEHLER:\n\tDas angegebene Skript existiert nicht.";
			requestResult = new ProtocolRequestResult(retVal);
		}
		if(!TEST) System.out.println(retVal);
		return retVal;
	}

	/**
	 * Liefert den Status der Protokollerstellung
	 *
	 * @return Ergebnis
	 */
	public String callGetStatus() {
		stayAlive = false;
		String retVal = "";
		System.out.println("FUEHRE \"ProtokollStatus\" AUS:");
		try {
			System.out.println("\tErmittelte Parameter:");
			long protocolId = new Long(this.parameter.get("protokollid"));
			System.out.println("\t\t- ProtokollId = " + protocolId);

			requestResult = new ProtocolRequestResult();
			byte status = client.getStatus(protocolId, requestResult);
			boolean erg = (status == -1) ? false : ((requestResult == null) ? true : requestResult.isRequestSuccessful());
			retVal = ("\tStatus:\t" + status);
			if(!erg) retVal += "\n\tFehler:\t\t" + requestResult.getErrorMsg();
		}
		catch(NullPointerException npEx) {
			retVal = "PARAMETERFEHLER:\n\tBenoetigte Parameter:" + "\n\t\t- ProtokollId";
			requestResult = new ProtocolRequestResult(retVal);
		}
		if(!TEST) System.out.println(retVal);
		return retVal;
	}

	/**
	 * Liefert den Status des Archivsystems und des PuA-Moduls
	 *
	 * @return Ergebnis
	 */
	public String callGetPuAStatus() {
		stayAlive = false;
		System.out.println("FUEHRE \"Status\" AUS:");

		requestResult = new ProtocolRequestResult();
		final boolean status = client.getIsActive(requestResult);
		final boolean erg = !status && (requestResult == null || requestResult.isRequestSuccessful());
		String result = ("\tStatus:\t" + status);
		if(!erg) result += "\n\tFehler:\t\t" + requestResult.getErrorMsg();

		if(!TEST) System.out.println(result);
		return result;
	}

	/**
	 * Stoppt die Verarbeitung bei der Protokollerstellung
	 *
	 * @return Ergebnis
	 * @deprecated Diese Methode war zur Flusskontrolle gedacht und wird nicht mehr benötigt, weil mittlerweile eine automatische Flusskontrolle zwischen
	 *             PuA-Server und -Client implementiert ist.
	 */
	@Deprecated
	public String callSuspendProtocol() {
		String retVal = "";
		System.out.println("FUEHRE \"ProtokollAnhalten\" AUS:");
		try {
			System.out.println("\tErmittelte Parameter:");
			long protocolId = new Long(this.parameter.get("protokollid"));
			System.out.println("\t\t- ProtokollId = " + protocolId);

			requestResult = client.suspendProtocol(protocolId);
			boolean erg = requestResult.isRequestSuccessful();
			retVal = "\tErfolgreich?\t" + (erg ? "Ja." : "Nein.");
			if(!erg) retVal += "\tFehler:\t\t" + requestResult.getErrorMsg();
		}
		catch(NullPointerException npEx) {
			retVal = "PARAMETERFEHLER:\n\tBenoetigte Parameter:" + "\n\t\t- ProtokollId";
			requestResult = new ProtocolRequestResult(retVal);
		}
		catch(FailureException fEx) {
			retVal = "\tFehler:\t\tProtokollerstellung anhalten fehlgeschlagen.";
			requestResult = new ProtocolRequestResult(retVal);
		}
		if(!TEST) System.out.println(retVal);
		return retVal;
	}

	/**
	 * Setzt die Verarbeitung bei der Protokollerstellung fort
	 *
	 * @return Ergebnis
	 * @deprecated Diese Methode war zur Flusskontrolle gedacht und wird nicht mehr benötigt, weil mittlerweile eine automatische Flusskontrolle zwischen
	 *             PuA-Server und -Client implementiert ist.
	 */
	@Deprecated
	public String callResumeProtocol() {
		String retVal = "";
		System.out.println("FUEHRE \"ProtokollFortsetzen\" AUS:");
		try {
			System.out.println("\tErmittelte Parameter:");
			long protocolId = new Long(this.parameter.get("protokollid"));
			System.out.println("\t\t- ProtokollId = " + protocolId);

			requestResult = client.resumeProtocol(protocolId);
			boolean erg = requestResult.isRequestSuccessful();
			retVal = "\tErfolgreich?\t" + (erg ? "Ja." : "Nein.");
			if(!erg) retVal += "\tFehler:\t\t" + requestResult.getErrorMsg();
		}
		catch(NullPointerException npEx) {
			retVal = "PARAMETERFEHLER:\n\tBenoetigte Parameter:" + "\n\t\t- ProtokollId";
			requestResult = new ProtocolRequestResult(retVal);
		}
		catch(FailureException fEx) {
			retVal = "\tFehler:\t\tProtokollerstellung fortsetzen fehlgeschlagen.";
			requestResult = new ProtocolRequestResult(retVal);
		}
		if(!TEST) System.out.println(retVal);
		return retVal;
	}

	/**
	 * Das letzte Ergebnis eines Methodenaufrufs auslesen
	 *
	 * @return das Ergebnis (z.B. die Pid des Skripts bei "createScript")
	 */
	public String getResultString() {
		return _resultString;
	}

	/**
	 * Der Stream zum Zugriff auf das letzte erzeugte Protokoll
	 *
	 * @return Stream des zuletzt erzeugten Protokolls.
	 */
	public ProtocolResultStream getStream() {
		return _stream;
	}

	/**
	 * Parameter setzen (nur benoetigt, falls CmdPua mit leerem Konstruktor instanziert wurde
	 *
	 * @param params Map
	 */
	public void setParameter(HashMap<String, String> params) {
		this.parameter = params;
	}

	/** Datenverteiler initialisieren */
	public void setDav() {
		if(dav == null) {
			try {
				printHeaderInfo("\nVERBINDE ZUM DATENVERTEILER: ");
				String userName = parameter.get("benutzer");
				String passwdFile = parameter.get("authentifizierung");
				String host = parameter.get("host");
				String port = parameter.get("port");
				String[] davParams = new String[]{
						"-datenverteiler=" + host + ":" + port, "-benutzer=" + userName, "-authentifizierung=" + passwdFile
				};
				dav = new ClientDavConnection(new ClientDavParameters(new ArgumentList(davParams)));
				dav.connect();
				dav.login();
				// Wenn TEST gesetzt ist, wird die Datenverteilerverbindung nicht immer beim cleanup() geschlossen, deshalb wird hier mit dem
				// NormalCloser dafür gesorgt, dass sich in diesem Fall der Test-Prozess nicht frühzeitig beendet, wenn die Verbindung zum Datenverteiler
				// verloren geht.
				if(TEST) dav.setCloseHandler(new NormalCloser());
				printHeaderInfo("OK (DAV-ZEIT: " + Util.timestrFormatted(dav.getTime()) + ")\n");
			}
			catch(Exception e) {
				printHeaderInfo("KEINE VERBINDUNG ZUM DATENVERTEILER (" + e.getMessage() + ")");
				requestResult = new ProtocolRequestResult("KEINE VERBINDUNG ZUM DATENVERTEILER (" + e.getMessage() + ")");
			}
			finally {
				printHeaderInfo("-----------------------------------------------------------------");
			}
		}
	}

	/** DatenverteilerDatenModell initialisieren */
	public void setDataModel() {
		if(model == null) model = dav.getDataModel();
	}

	/**
	 * Konfigurations-Authoritaet setzen
	 *
	 * @throws IllegalArgumentException Exception
	 */
	public void setConfigAuthority() throws IllegalArgumentException {
		if(parameter.get("konfverantwortlicher") != null) {
			if(dav.getDataModel().getObject(
					parameter.get("konfverantwortlicher")
			) instanceof ConfigurationAuthority) {
				confAuth = (ConfigurationAuthority)dav.getDataModel()
						.getObject(parameter.get("konfverantwortlicher"));
			}
			else {
				throw new IllegalArgumentException("Konfigurationsverantwortlicher " + parameter.get("konfverantwortlicher") + " existiert nicht.");
			}
		}
		else {
			confAuth = dav.getLocalConfigurationAuthority();
		}
	}

	/**
	 * Erzeugt einen neuen PuaClient. Dieser kann über {@link #client} abgerufen werden.
	 *
	 * @throws ScopeException Exception
	 */
	public void setClient() throws ScopeException {
		client = new PuaClient(dav, confAuth);
	}

	/** Aufraeumen (Datenverteiler-Verbindung schliessen) */
	public void cleanUp() {
		if(dav != null && !stayAlive) dav.disconnect(false, "Bye.");
	}

	/** alle Parameter ausgeben */
	public void printParams() {
		Set<String> keys = parameter.keySet();
		System.out.println("GEFUNDENE PARAMTETER:");
		for(Iterator<String> iter = keys.iterator(); iter.hasNext();) {
			String element = (String)iter.next();
			String tab = "";
			for(int i = (element.length() + 1) / 8; i < 4; i++) {
				tab += "\t";
			}
			System.out.println("\t" + element + "=" + tab + parameter.get(element));
		}
		System.out.println("-----------------------------------------------------------------");
	}

	/**
	 * Gibt einen Stream aus.
	 *
	 * @param protocolResultStream ProtocolResultStream
	 */
	public void printStream(ProtocolResultStream protocolResultStream) {
		boolean first = true;
		ProtocolViewer pv = null;
		if(_gui) pv = new ProtocolViewer(model);
		try {
			// display protocol-result
			int c = 0;
			int numData = 1;
			while(protocolResultStream.hasNext()) {
				ProtocolResult res = protocolResultStream.take();
				final Long id = protocolResultStream.getJobId();
				if(id != null) mostRecentProtocolId = id;
				if(!_gui) {
					if(res instanceof ProtocolAdministrationData && _printHeader) {
						System.out.println("Protokoll-ID: " + protocolResultStream.getJobId());
						ProtocolAdministrationData adminData = (ProtocolAdministrationData)res;
						System.out.print("\t");
						for(int i = 0; i < adminData.columns.length; i++) {
							System.out.print(
									adminData.columns[i].getCaption() + "\t"
							);
						}
						System.out.print("[NAME]\n");
						System.out.println("");
						System.out.println("Ersteller: " + adminData.creatorName);
						System.out.println("Datum:     " + Util.msToDate(adminData.protocolDate));
						System.out.println("Beschreibung:\n" + adminData.infoText);
						System.out.println("");
					}
					if(res instanceof ProtocolAggregationData && _printHeader) {
						ProtocolAggregationData aggregData = (ProtocolAggregationData)res;
						System.out.print("\t");
						for(final List<String> aggregations : aggregData.getAggregations()) {
							System.out.print(aggregations + "\t");
						}
						System.out.print("[AGGREGATIONEN]\n");
						System.out.print("\t");
						for(final ProcessingInformation.ApplyAggregation applyAggregation : aggregData.getOrders()) {
							System.out.print(applyAggregation + "\t");
						}
						System.out.print("[ANWENDUNGEN]\n");
					}
					if(res instanceof ProtocolNoChanges) {
						ProtocolNoChanges noChangeData = (ProtocolNoChanges)res;
						System.out.println((c++) + " No Change " + noChangeData.timestamp);
					}
					if(res instanceof ProtocolResultData) {
						final ProtocolResultData resultData = (ProtocolResultData)res;
						System.out.print("#" + numData + ":\t");
						System.out.print(Util.msToDate(resultData.getTimeStamp()) + ":\t");
						System.out.print(resultData.getIntervalIndex() + "\t");
						for(final ProtocolData protocolData : resultData.getDataList(model)) {
							System.out.print(protocolData.valueToString() + "\t");
						}
						numData++;
						System.out.println("");						
					}
					if(res instanceof ProtocolAggregationResultData) {
						final ProtocolAggregationResultData resultData = (ProtocolAggregationResultData)res;
						System.out.print(Util.msToDate(resultData.getStartTimeStamp()) + "-");
						System.out.print(Util.msToDate(resultData.getEndTimeStamp()) + ":\t");
						//System.out.print(resultData.getIntervalIndex() + "\t");
						for(final ProtocolData protocolData : resultData.getDataList(model)) {
							System.out.print(protocolData.valueToString() + "\t");
						}
						System.out.print(resultData.order);
						System.out.println("");
					}
					if(res == null && _printHeader) {
						System.out.println("end of stream.");
					}
				}
				else {
					pv.add(res);
					//Anzeige in einer Tabelle
					if(first) {
						if(protocolResultStream.getJobId() != null) pv.setProtocolId(protocolResultStream.getJobId());
						first = false;
					}
				}
			}
			if(!protocolResultStream.isComplete())	//ggf Fehler ausgeben:
			{
				if(_gui && pv.isReady()) {
					pv.showErrorMessage(protocolResultStream.getErrorMsg());
				}
				else {
					System.out.println("\nEs ist ein Fehler aufgetreten: " + protocolResultStream.getErrorMsg());
				}
			}
		}
		catch(InterruptedException irEx) {
			System.out.println("Fehler beim Lesen des Antwortstreams (" + irEx.getMessage() + ").");
			requestResult = new ProtocolRequestResult("Fehler beim Lesen des Antwortstreams (" + irEx.getMessage() + ").");
		}
	}


	/** Gibt Informationen über die Kommandozeilenparameter aus. */
	public void printInfo() {
		//wenn diese Methode aufgerufen wird, dann gab es wohl einen Fehler
		System.out.println("");
		System.out.println("MOEGLICHE OPERATIONEN:");
		System.out.println("\tStatus");
		System.out.println("\tSkriptAnlegen");
		System.out.println("\tSkriptQuellcode");
		System.out.println("\tSkriptStandardwerte");
		System.out.println("\tSkriptLoeschen");
		System.out.println("\tProtokollErstellen");
		System.out.println("\tProtokollAbbrechen");
		System.out.println("\tProtokollLoeschen");
		System.out.println("\tProtokollStatus");
		System.out.println("\tProtokollAnhalten (@Deprecated)");
		System.out.println("\tProtokollFortsetzen (@Deprecated)");
		System.out.println("\tGespeichertesProtokoll");
		System.out.println("\tUngelesenesProtokoll");
		System.out.println("\tListeGespeicherteProtokolle");
		System.out.println("\tListeUngeleseneProtokolle");
		System.out.println("");

		System.out.println("OBLIGATORISCHE PARAMETER:");
		System.out.println("\tOperation=[String] -> die Methode, die aufgerufen wird (siehe MOEGLICHE OPERATIONEN)");
		System.out.println("\tKonfVerantwortlicher=[String] -> PID des Konfigurationsverantwortlichen (optional, Standard ist lokal Verantwortl.)");
		System.out.println("\tHost=[String] -> Host auf dem der Datenverteiler laeuft");
		System.out.println("\tPort=[String] -> Port des Datenverteilers");
		System.out.println("\tBenutzer=[String] -> Benutzername für Login beim Datenverteiler");
		System.out.println("\tAuthentifizierung=[String] -> Dateiname der Datei, die Authentifikationsdaten für den Datenverteiler-Login enthaelt");
		System.out.println("\tTimeout=[Long] -> Maximale Wartezeit in ms, bis eine Antwort eingetroffen sein muss (optional)");
		System.out.println("\tTabelle=[ja|nein] -> Anzeige der Protokolle in einer Tabelle oder Ausgabe auf der Konsole.(optional- Defaultwert 'nein')");
		System.out.println("\tKopf=[ja|nein] -> Kopf des Protokolls anzeigen. Nur in Verbindung mit Tabelle=nein. Parameter ist optional, Defaultwert 'ja')");
		System.out.println("");

		System.out.println("PARAMETER FUER OPERATIONEN:");
		System.out.println("\tDatenarten=[1|2|3|4][,1|2|3|4]* -> 1:Online, 2:Online-verzögert, 3:Nachgefprdert, 4:Nachgefordert-verzögert (kommagetrennt)<br>");
		System.out.println("\tBindungen=[String]=[String]{,[String]=[String]}* -> Bindungen der freien Aspekte");
		System.out.println("\tDeltaAnfrage=[ja|nein]");
		System.out.println("\tHauptobjekt=[PID des Hauptobjekts]");
		System.out.println("\tObjekte=[PIDs von anderen Objekten (kommagetrennt)]");
		System.out.println("\tZeitbereich=Long-Long[,Long-Long]* -> Zeitbereiche");
		System.out.println("\tPseudoObjekte=([String]{,[String]}*n) = {([String]{,[String]}*n)}* -> Definitionen der Pseudoobjekte");
		System.out.println("\tSpeichern=[ja|nein]");
		System.out.println("\tSkript=[PID des Skriptobjekts]");
		System.out.println("\tSimVar=[Short] -> Simulationsvariante");
		System.out.println("\tProtokollId=[Long] -> Id des Protokolls");
		System.out.println("\tQuelltext=[String] -> Dateiname einer Datei, die den Quelltext eines PuA-Skripts enthaelt");
		System.out.println("\tName=[String] -> Name des Skripts (optional fuer createScript)");
		System.out.println("\tBeschreibung=[String] -> Beschreibung (optional fuer createScript)");
		System.out.println("\tAutor=[String] -> Author (optional fuer createScript)");
		System.out.println("\tStatus=[String] -> Status (optional fuer createScript)");
		System.out.println("\tDatum=[String] -> Datum (optional fuer createScript)");
		System.out.println("");
		printParams();
	}

	private String hash2String(Hashtable<String, String[]> pseudoObjects) {
		String psObjStr = "";
		Set<String> keys = pseudoObjects.keySet();
		for(Iterator<String> iter = keys.iterator(); iter.hasNext();) {
			System.out.println();
			String objalias = (String)iter.next();
			psObjStr += "(" + objalias + "=";
			for(String tmp1 : pseudoObjects.get(objalias)) {
				psObjStr += tmp1 + ",";
			}
			psObjStr = psObjStr.substring(0, psObjStr.length() - 1) + ") ";
		}
		return psObjStr.substring(0, psObjStr.length() - 1) + "]";
	}
}

