/*
 * Copyright 2010 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.manager.status;

import de.bsvrz.pua.prot.manager.taskmanager.ThreadElement;
import de.bsvrz.pua.prot.manager.taskmanager.ThreadPool;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Veröffentlicht den Status der Protokolle über die Konsole
 *
 * @author Kappich Systemberatung
 * @version $Revision: 0000 $
 */
public class ConsolePublisher extends StatePublisherTask {

	/** Ausgabe */
	private static final Debug _debug = Debug.getLogger();

	@Override
	public int getInterval() {
		return 60 * 1000;
	}

	@Override
	protected void update(final StatePublisher.Status status) {
		final StringBuilder builder = new StringBuilder();
		builder.append("Informationen über Protokolle:\n");
		builder.append("  Anfragen in Warteschlange: ").append(status.getQueuedThreads().size()).append("\n");
		for(final ThreadPool.QueuedThread queuedThread : status.getQueuedThreads()) {
			builder.append("    Protokoll-ID: ").append(queuedThread.getProtocolId()). append(" ");
			builder.append("    Auftrags-ID: ").append(queuedThread.getJobId()). append(" ");
			builder.append(queuedThread.getOperation()). append("\n");
		}
		builder.append("  Aktuelle Threads: ").append(status.getActiveThreads().size()).append("\n");
		for(final ThreadElement threadElement : status.getActiveThreads()) {
			builder.append("    Protokoll-ID: ").append(threadElement.getProtocolId()). append(" ");
			builder.append("    Auftrags-ID: ").append(threadElement.getJobId()). append(" ");
			builder.append(threadElement.getOperation()). append(" ");
			builder.append(threadElement.getStatus()).append("%\n");
		}
		_debug.info(builder.toString());
	}

	@Override
	public void notifyThreadFinished(final ThreadElement threadElement) {
		final StringBuilder builder = new StringBuilder();
		builder.append("Protokoll fertig:\n");
		builder.append("    Protokoll-ID: ").append(threadElement.getProtocolId()). append(" ");
		builder.append("    Auftrags-ID: ").append(threadElement.getJobId()).append(" ");
		builder.append(threadElement.getOperation()). append(" ");
		_debug.info(builder.toString());
	}

	@Override
	public void notifyProtocolRemoved(final long protocolId) {
		final StringBuilder builder = new StringBuilder();
		builder.append("Protokoll gelöscht:\n");
		builder.append("    Protokoll-ID: ").append(protocolId);
		_debug.info(builder.toString());
	}

	@Override
	public void notifyProtocolRead(final long protocolId) {
		final StringBuilder builder = new StringBuilder();
		builder.append("Protokoll gelesen:\n");
		builder.append("    Protokoll-ID: ").append(protocolId);
		_debug.info(builder.toString());
	}

	@Override
	public void notifyArchiveAvailable(final boolean archiveAvailable) {
		_debug.info(archiveAvailable ? "Das Archivsystem ist jetzt verfügbar" : "Das Archivsystem ist nicht mehr verfügbar");
	}
}
