/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung - Bibliothek
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.DavConnectionListener;

/**
 * Klasse mit den Methoden für den Zugriff auf die Parametrierung der
 * Kernsoftware.
 *
 * @author BitCtrl Systems GmbH, Peuker
 * @version $Id$
 */
public final class MethodenBibliothek {

	static final long MILLIS_PER_MINUTE = 60 * 1000;

	/**
	 * Listener zur Auswertung von Verbindungsabbrüchen zum Datenverteiler.<br>
	 */
	private static final DavConnectionListener LISTENER = new DavConnectionListener() {

		/**
		 * {@inheritDoc}
		 *
		 * entfernt alle Objekte, die mit der geschlossenen Verbindung
		 * assoziiert sind.
		 */
		@Override
		public void connectionClosed(final ClientDavInterface connection) {
			MethodenBibliothek.bibliothekAufraeumen(connection);
		}
	};

	/** die Liste der Parametermanager auf Verbindungen bezogen. */
	private static final Map<ClientDavInterface, ParameterManager> PARAMETER_MANAGER_LISTE = new HashMap<>();

	/** die Liste der verwendeten Datenverteilerverbindungen. */
	private static final Collection<ClientDavInterface> VERBINDUNGEN = new ArrayList<>();

	/**
	 * bereinigt die Listen für Parametermanager und Hierarchieobjekte.
	 *
	 * @param verbindung
	 *            die Verbindung, die nicht mehr unterstützt werden soll.
	 */
	static void bibliothekAufraeumen(final ClientDavInterface verbindung) {
		final ParameterManager paraManager = MethodenBibliothek.PARAMETER_MANAGER_LISTE.get(verbindung);
		if (null != paraManager) {
			paraManager.ungueltigSetzen();
			MethodenBibliothek.PARAMETER_MANAGER_LISTE.remove(verbindung);
		}

		MethodenBibliothek.VERBINDUNGEN.remove(verbindung);
	}

	/**
	 * liefert das Hierarchie-Objekt für die übergebene
	 * Datenverteilerverbindung.
	 *
	 * @param verbindung
	 *            die Verbindung
	 * @return das Hierarchie-Objekt
	 */
	public static Hierarchie getHierarchie(final ClientDavInterface verbindung) {
		return MethodenBibliothek.getParameterManager(verbindung);
	}

	/**
	 * liefert den Parametermanager für die übergebene Datenverteilerverbindung.
	 *
	 * @param verbindung
	 *            die Verbindung
	 * @return der Parametermanager
	 */
	public static ParameterManager getParameterManager(final ClientDavInterface verbindung) {
		MethodenBibliothek.registriereVerbindung(verbindung);
		ParameterManager manager = MethodenBibliothek.PARAMETER_MANAGER_LISTE.get(verbindung);
		if (null == manager) {
			manager = new ParameterManager(verbindung);
			MethodenBibliothek.PARAMETER_MANAGER_LISTE.put(verbindung, manager);
		}
		return manager;
	}

	/**
	 * fügt eine verwaltete Verbindung hinzu.
	 *
	 * @param verbindung
	 *            die Verbindung
	 */
	private static void registriereVerbindung(final ClientDavInterface verbindung) {
		if (!MethodenBibliothek.VERBINDUNGEN.contains(verbindung)) {
			MethodenBibliothek.VERBINDUNGEN.add(verbindung);
			verbindung.addConnectionListener(MethodenBibliothek.LISTENER);
		}
	}

	/**
	 * Privater Konstruktor, verhindert die Erstellung einer Instanz der Klasse.
	 */
	private MethodenBibliothek() {
		super();
	}
}
