/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung - Bibliothek
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Repräsentation der Quelle eines Parameters.<br>
 * Die Quelle beschreibt, an welchem Objekt der Wert des Parameters ermittelt
 * wurde und um welche Art von Parameter (lokal, global, Standard) es sich
 * handelt.
 *
 * @author BitCtrl Systems GmbH, Peuker
 * @version $Id$
 */
public class Quelle {

	/**
	 * das Datenverteiler-Sytemobjekt, an dem die Daten eines Parameters
	 * ermittelt wurden.
	 */
	private SystemObject objekt;

	/** der Typ der Quelle. */
	private QuellTyp typ;

	/** sind Daten verfügbar ? */
	private boolean dataAvailable;

	/**
	 * erzeugt eine Intsanz einer Quellendefinition mit den übergebenen Daten
	 * für den Typ und das Systemobjekt.
	 *
	 * @param objekt
	 *            das Systemobjekt
	 * @param typ
	 *            der Typ der Quelle
	 */
	Quelle(final SystemObject objekt, final QuellTyp typ) {
		this.objekt = objekt;
		this.typ = typ;
	}

	/**
	 * liefert das Systemobjekt, an dem die Quelldaten ermittelt wurden.
	 *
	 * @return das Systemobjekt
	 * @see #setObjekt(SystemObject)
	 */
	public SystemObject getObjekt() {
		return objekt;
	}

	/**
	 * erzeugt eine serialisierbare Form des Quellobjekts zur Übertragung in
	 * einem Byteaarray innerhalb eines Datenverteiler-Datensatzes.
	 *
	 * @return das serialisierbare Objekt
	 */
	SerializableQuelle getSerializable() {
		return new SerializableQuelle(this);
	}

	/**
	 * liefert den Typ der Quelle.
	 *
	 * @return den Typ
	 * @see #setTyp(QuellTyp)
	 */
	public QuellTyp getTyp() {
		return typ;
	}

	/**
	 * ermittelt, ob die Quelle potentiell Daten liefern kann.
	 *
	 * @return der Zustand
	 */
	public boolean isDataAvailable() {
		return dataAvailable;
	}

	/**
	 * definiert, ob die Quelle potentiell Daten liefern kann.
	 *
	 * @param dataAvailable
	 *            der Zustand
	 */
	public void setDataAvailable(final boolean dataAvailable) {
		this.dataAvailable = dataAvailable;
	}

	/**
	 * setzt das Objekt, an dem die Quelldaten ermittelt wurden.
	 *
	 * @param objekt
	 *            das Systemobjekt
	 * @see #getObjekt()
	 */
	void setObjekt(final SystemObject objekt) {
		this.objekt = objekt;
	}

	/**
	 * setzt den Typ der Quelle.
	 *
	 * @param typ
	 *            der Typ
	 * @see #getTyp()
	 */
	public void setTyp(final QuellTyp typ) {
		this.typ = typ;
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		final StringBuffer buffer = new StringBuffer(100);
		buffer.append(typ);
		buffer.append(": ");
		buffer.append(objekt);
		return buffer.toString();
	}
}
